<?php

namespace Drupal\drd_pi_acquia\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\drd_pi_acquia\Entity\Account;
use GuzzleHttp\Client;

/**
 * Class Settings.
 */
class Settings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      Account::getConfigName(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'drd_pi_acquia_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(Account::getConfigName());

    $form['endpoint'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API endpoint'),
      '#default_value' => $config->get('endpoint'),
      '#required' => TRUE,
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * @inheritDoc
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    $endpoint = $form_state->getValue('endpoint');
    try {
      $client = new Client(['base_uri' => $endpoint]);
      $response = $client->request('head');
      if (!$response->hasHeader('X-Rate-Limit-Limit')) {
        $form_state->setErrorByName('endpoint', $this->t('This is not an Acquia Cloud API endpoint.'));
      }
    }
    catch (\Exception $ex) {
      $form_state->setErrorByName('endpoint', $this->t('Can not connect to that endpoint.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $this->config(Account::getConfigName())
      ->set('endpoint', $form_state->getValue('endpoint'))
      ->save();
  }

}
