<?php

namespace Drupal\drd\Agent\Action;

/**
 * Base class for Remote DRD Action Code.
 */
abstract class Base implements BaseInterface {

  private static $debugMode = FALSE;
  private static $arguments = array();
  /** @var \Drupal\drd\Crypt\BaseMethodInterface */
  protected static $crypt;

  /**
   * @param $items
   * @return mixed
   */
  private static function toArray($items) {
    foreach ($items as $key => $item) {
      if (is_object($item)) {
        $items[$key] = self::toArray((array) $item);
      }
      elseif (is_array($item)) {
        $items[$key] = self::toArray($item);
      }
    }
    return $items;
  }

  public static function run($version, $debugMode = FALSE) {
    self::setDebugMode($debugMode);
    require_once DRD_BASE . "/Action/V$version/Base.php";
    require_once DRD_BASE . "/Auth/V$version/Base.php";
    \Drupal\drd\Agent\Remote\Base::loadClasses($version);
    /** @var BaseInterface $self */
    $self = "\\Drupal\\drd\\Agent\\Action\\V$version\\Base";

    try {
      $raw_input = file_get_contents('php://input');
      if (empty($raw_input)) {
        throw new \Exception('Can not read input');
      }

      $input = unserialize(base64_decode($raw_input));
      if (!is_array($input) || empty($input)) {
        throw new \Exception('Input is empty');
      }

      if (empty($input['uuid']) || empty($input['args']) || !isset($input['iv'])) {
        throw new \Exception('Input is incomplete');
      }

      if (!empty($input['ott']) && !empty($input['config'])) {
        if (!$self::ott($input['ott'], $input['config'])) {
          throw new \Exception('OTT config failed');
        }
        return 'ok';
      }

      self::$crypt = $self::getCryptInstance($input['uuid']);
      if (!self::$crypt) {
        throw new \Exception('Encryption method not available or unauthorised');
      }
      $args = self::toArray(self::$crypt->decrypt($input['args'], $input['iv']));

      if (empty($args['auth']) || !isset($args['authsetting']) || empty($args['action'])) {
        throw new \Exception('Arguments incomplete');
      }

      $auth_methods = \Drupal\drd\Agent\Auth\Base::getMethods($version);
      if (!isset($auth_methods[$args['auth']]) || !($auth_methods[$args['auth']] instanceof \Drupal\drd\Agent\Auth\BaseInterface)) {
        throw new \Exception('Unrecognized authentication method');
      }

      /** @var \Drupal\drd\Agent\Auth\BaseInterface $auth */
      $auth = $auth_methods[$args['auth']];
      if (!$auth->validateUUID($input['uuid'])) {
        throw new \Exception('DRD instance not registered');
      }
      if (!$auth->validate($args['authsetting'])) {
        throw new \Exception('Not authenticated');
      }

      $action = $args['action'];
      unset($args['auth']);
      unset($args['authsetting']);
      unset($args['action']);
      self::$arguments = $args;
    }
    catch (\Exception $ex) {
      $self::watchdog($ex->getMessage(), array(), 3);
      header("HTTP/1.1 502 Error");
      print 'error';
      exit;
    }

    try {
      $self::promoteUser();

      require_once DRD_BASE . "/Action/V$version/$action.php";
      $classname = "\\Drupal\\drd\\Agent\\Action\\V$version\\$action";

      /** @var BaseInterface $actionObject */
      $actionObject = new $classname;
    }
    catch (\Exception $ex) {
      $self::watchdog('Not yet implemented: ' . $action, array(), 3);
      header("HTTP/1.1 403 Not found");
      print 'Not yet implemented';
      exit;
    }

    $result = $actionObject->execute();
    if (is_array($result)) {
      $result['messages'] = drupal_get_messages();
      return base64_encode(self::$crypt->encrypt($result));
    }
    return $result;
  }

  /**
   * @inheritDoc
   */
  public static function getArguments() {
    return self::$arguments;
  }

  /**
   * @inheritDoc
   */
  public static function getDebugMode() {
    return self::$debugMode;
  }

  /**
   * @inheritDoc
   */
  public static function setDebugMode($debugMode) {
    self::$debugMode = $debugMode;
  }

}
