<?php

namespace Drupal\drd\Command;

use Drupal\Console\Command\Shared\ConfirmationTrait;
use Drupal\Console\Command\Shared\FormTrait;
use Drupal\Console\Command\Shared\ModuleTrait;
use Drupal\Console\Core\Command\Shared\CommandTrait;
use Drupal\Console\Core\Style\DrupalStyle;
use Drupal\drd\Plugin\Action\BaseInterface;
use Drupal\user\Entity\User;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class Base.
 *
 * @package Drupal\drd
 */
abstract class Base extends Command {

  protected $action_key;

  /** @var \Drupal\drd\Logging */
  protected $logger;

  use CommandTrait;
  use ModuleTrait;
  use FormTrait;
  use ConfirmationTrait;

  protected function promoteUser() {
    \Drupal::currentUser()->setAccount(User::load(1));
  }

  /**
   * {@inheritdoc}
   */
  protected function configure() {
  }

  /**
   * {@inheritdoc}
   */
  protected function execute(InputInterface $input, OutputInterface $output) {
    $io = new DrupalStyle($input, $output);
    $this->logger = \Drupal::service('drd.logging');
    $this->logger->setIO($io);
    if ($io->isDebug()) {
      $this->logger->enforceDebug();
    }
    $this->promoteUser();

    $action = \Drupal\drd\Plugin\Action\Base::instance($this->action_key);
    if (!$action || !($action instanceof BaseInterface)) {
      $io->error('No valid action!');
      return FALSE;
    }

    $this->setActionArguments($action, $input);
    return $action;
  }

  /**
   * Callback to collect all values from the command line and pass them on to
   * the action before that gets executed.
   *
   * @param \Drupal\drd\Plugin\Action\BaseInterface $action
   * @param \Symfony\Component\Console\Input\InputInterface $input
   */
  protected function setActionArguments($action, InputInterface $input) {
  }

}
