<?php

namespace Drupal\drd\Command;

use Drupal\Console\Core\Style\DrupalStyle;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class BaseEntity.
 *
 * @package Drupal\drd
 */
abstract class BaseEntity extends Base {

  /**
   * @param \Symfony\Component\Console\Input\InputInterface $input
   * @param \Drupal\Console\Core\Style\DrupalStyle $io
   * @param \Drupal\drd\Plugin\Action\BaseEntityRemote $action
   *
   * @return bool|\Drupal\drd\Entity\BaseInterface[]
   */
  protected abstract function getEntities($input, $io, $action);

  /**
   * {@inheritdoc}
   */
  protected function configure() {
    parent::configure();
    $this
      ->addOption(
        'host-id',
        NULL,
        InputOption::VALUE_OPTIONAL,
        $this->trans('commands.drd.actions.remote.arguments.host-id')
      )
      ->addOption(
        'core-id',
        NULL,
        InputOption::VALUE_OPTIONAL,
        $this->trans('commands.drd.actions.remote.arguments.core-id')
      )
      ->addOption(
        'domain-id',
        NULL,
        InputOption::VALUE_OPTIONAL,
        $this->trans('commands.drd.actions.remote.arguments.domain-id')
      );
  }

  /**
   * {@inheritdoc}
   */
  protected function execute(InputInterface $input, OutputInterface $output) {
    /** @var \Drupal\drd\Plugin\Action\BaseEntityRemote $action */
    $action = parent::execute($input, $output);
    $io = new DrupalStyle($input, $output);
    if (empty($action)) {
      $io->error('No action found!');
      return;
    }

    $entities = $this->getEntities($input, $io, $action);
    if (!isset($entities)) {
      $io->error('No valid action found!');
      return;
    }
    if (empty($entities)) {
      $io->error('No entities found!');
      return;
    }

    $io->info('Executing ' . $action->getPluginDefinition()['label']);
    foreach ($entities as $entity) {
      /** @var \Drupal\drd\Entity\BaseInterface $entity */
      $io->info('- on id ' . $entity->id() . ': ' . $entity->getName());
      if ($action->executeAction($entity)) {
        $io->success('  ok!');
      }
      else {
        $io->error('  failure!');
      }

      $output = $action->getOutput();
      if ($output) {
        foreach ($output as $value) {
          $io->info('  ' . $value);
        }
      }
    }

    $io->info('Completed!');
  }

}
