<?php

namespace Drupal\drd\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Routing\TrustedRedirectResponse;

/**
 * Class Domain.
 *
 * @package Drupal\drd\Controller
 */
class Domain extends ControllerBase {

  /**
   * Return from remote after initially setting the configuration to then
   * retrieve core details and all other hosted domains.
   *
   * @var \Drupal\drd\Entity\DomainInterface $domain
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   */
  public function returnFromRemote($domain) {
    $core = $domain->getCore();
    $domain->set('installed', 1)->save();

    // Get info from remote
    $domain->initCore($core);

    // Get all remote domains
    $domain->retrieveAllDomains($core);

    return $this->redirect('entity.drd_core.canonical', ['drd_core' => $core->id()]);
  }

  /**
   * Redirect to the remote domain by opening a new session.
   *
   * @var \Drupal\drd\Entity\DomainInterface $domain
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   */
  public function session($domain) {
    $url = $domain->getSessionUrl();
    if (!$url) {
      drupal_set_message('Can not retrieve login URL from remote domain.', 'error');
      return $this->redirect('entity.drd_domain.canonical', ['drd_domain' => $domain->id()]);
    }
    return new TrustedRedirectResponse($url);
  }

}
