<?php

namespace Drupal\drd\Crypt\Method;

use Drupal\Core\Form\FormStateInterface;
use Drupal\drd\Crypt\BaseMethod;

/**
 * Provides security over TLS without additional encryption.
 *
 * @ingroup drd
 */
class Tls extends BaseMethod {

  public function __construct($settings = array()) {

  }

  /**
   * @inheritDoc
   */
  public function requiresPassword() {
    return FALSE;
  }

  /**
   * @inheritDoc
   */
  public function getLabel() {
    return 'TLS';
  }

  /**
   * @inheritDoc
   */
  public function getCipher() {
    return FALSE;
  }

  /**
   * @inheritDoc
   */
  public function getPassword() {
    return FALSE;
  }

  /**
   * @inheritDoc
   */
  public function isAvailable() {
    try {
      if (function_exists('module_exists')) {
        // We are not on D8
      }
      elseif (\Drupal::moduleHandler()->moduleExists('drd')) {
        // We are on the dashboard and not remote, so here we do support TLS.
        return TRUE;
      }
    }
    catch (\Exception $ex) {

    }
    if (function_exists('variable_get')) {
      // This is on Drupal 7 or earlier
      return variable_get('https', FALSE);
    }
    return \Drupal::config('https');
  }

  /**
   * @inheritDoc
   */
  public function getCipherMethods() {
    return array();
  }

  /**
   * @inheritDoc
   */
  public function settingsForm(array &$form, array $condition) {
    $form['info'] = array(
      '#type' => 'item',
      '#markup' => t('No settings required.'),
      '#states' => array(
        'required' => $condition,
      ),
    );
  }

  /**
   * @inheritDoc
   */
  public function settingsFormValues(FormStateInterface $form_state) {
    return $this->getSettings();
  }

  /**
   * @inheritDoc
   */
  public function getIV() {
    return '';
  }

  /**
   * @inheritDoc
   */
  public function encrypt($args) {
    return serialize($args);
  }

  /**
   * @inheritDoc
   */
  public function decrypt($body, $iv) {
    return unserialize($body);
  }

}
