<?php

namespace Drupal\drd\Entity;

/**
 * Provides an interface for defining Core entities.
 *
 * @ingroup drd
 */
interface CoreInterface extends BaseInterface {

  /**
   * @return string
   */
  public function getLangCode();

  /**
   * Gets the Core host.
   *
   * @return HostInterface
   *   Host of the Core.
   */
  public function getHost();

  /**
   * Sets the Core drupal release.
   *
   * @param ReleaseInterface $release
   *   The Core drupal release.
   *
   * @return $this
   *   The called Core entity.
   */
  public function setDrupalRelease($release);

  /**
   * Gets the Core drupal release.
   *
   * @return ReleaseInterface
   *   Drupal release of the Core.
   */
  public function getDrupalRelease();

  /**
   * @return array
   */
  public function getUpdateSettings();

  /**
   * @return \Drupal\drd\Update\PluginStorageInterface
   */
  public function getUpdatePlugin();

  /**
   * Sets the Core host.
   *
   * @param HostInterface $host
   *   The Core host.
   *
   * @return $this
   *   The called Core entity.
   */
  public function setHost($host);

  /**
   * @param array $properties
   * @return DomainInterface[]
   */
  public function getDomains($properties = []);

  /**
   * @return DomainInterface
   */
  public function getFirstActiveDomain();

  /**
   * @return ReleaseInterface[]
   */
  public function getAvailableUpdates();

  /**
   * @return array
   */
  public function getUpdateLogList();

  /**
   * @param int $timestamp
   *
   * @return string
   */
  public function getUpdateLog($timestamp);

  /**
   * @param string $log
   *
   * @return $this
   */
  public function saveUpdateLog($log);

}
