<?php

namespace Drupal\drd\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Defines the Script Type entity.
 *
 * @ConfigEntityType(
 *   id = "drd_script_type",
 *   label = @Translation("Script Type"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\drd\Entity\ListBuilder\ScriptType",
 *
 *     "form" = {
 *       "default" = "Drupal\drd\Entity\Form\ScriptType",
 *       "add" = "Drupal\drd\Entity\Form\ScriptType",
 *       "edit" = "Drupal\drd\Entity\Form\ScriptType",
 *       "delete" = "Drupal\drd\Entity\Form\ScriptTypeDelete"
 *     },
 *   },
 *   config_prefix = "script_type",
 *   admin_permission = "drd.administer script entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid"
 *   },
 *   links = {
 *     "canonical" = "/drd/settings/script-types/script-type/{drd_script_type}",
 *     "add-form" = "/drd/settings/script-types/add",
 *     "edit-form" = "/drd/settings/script-types/script-type/{drd_script_type}/edit",
 *     "delete-form" = "/drd/settings/script-types/script-type/{drd_script_type}/delete",
 *     "collection" = "/drd/settings/script-types"
 *   }
 * )
 */
class ScriptType extends ConfigEntityBase implements ScriptTypeInterface {

  private static $selectList;

  public static function getSelectList() {
    if (!isset(self::$selectList)) {
      self::$selectList = [
        '' => '-',
      ];
      $config = \Drupal::config('drd.script_type');
      foreach ($config->getStorage()->listAll('drd.script_type') as $key) {
        $script = $config->getStorage()->read($key);
        self::$selectList[$script['id']] = $script['label'];
      }
    }
    return self::$selectList;
  }

  /** @var string */
  protected $id;

  /** @var string */
  protected $label;

  /** @var string */
  protected $interpreter;

  /** @var string */
  protected $extension;

  /** @var string */
  protected $prefix;

  /** @var string */
  protected $suffix;

  /** @var string */
  protected $lineprefix;

  /**
   * @inheritDoc
   */
  public function interpreter() {
    return $this->interpreter;
  }

  /**
   * @inheritDoc
   */
  public function extension() {
    return $this->extension;
  }

  /**
   * @inheritDoc
   */
  public function prefix() {
    return $this->prefix;
  }

  /**
   * @inheritDoc
   */
  public function suffix() {
    return $this->suffix;
  }

  /**
   * @inheritDoc
   */
  public function lineprefix() {
    return $this->lineprefix;
  }

}
