<?php

namespace Drupal\drd;

/**
 * Class Logging.
 *
 * @package Drupal\drd
 */
class Logging {

  protected $debug = FALSE;

  /** @var \Symfony\Component\Console\Style\SymfonyStyle */
  protected $io;

  /**
   * @inheritDoc
   */
  public function __construct() {
    $this->debug = \Drupal::config('drd.general')->get('debug');
  }

  public function enforceDebug() {
    $this->debug = TRUE;
  }

  /**
   * @param \Symfony\Component\Console\Style\SymfonyStyle $io
   */
  public function setIO($io) {
    $this->io = $io;
  }

  /**
   * @param string $severity
   * @param string $message
   * @param array $args
   */
  public function log($severity, $message, $args = []) {
    if (!method_exists(\Drupal::logger('drd'), $severity)) {
      $severity = 'emergency';
    }
    $arguments = [];

    $plugin_available = isset($args['@plugin_id']);
    $entity_available = isset($args['@entity_type']);

    if ($plugin_available && $entity_available) {
      $message = '@plugin_id [@entity_type/@entity_id]: ' . $message;
    }
    elseif ($plugin_available) {
      $message = '@plugin_id: ' . $message;
    }
    elseif ($entity_available) {
      $message = '[@entity_type/@entity_id]: ' . $message;
    }

    $loggerMessage = '';
    foreach ($args as $arg => $value) {
      if ($arg == 'link' || strpos($message, $arg) !== FALSE) {
        $arguments[$arg] = $value;
      }
      else {
        if (is_scalar($value)) {
          $message .= ' ' . $arg;
          $arguments[$arg] = $value;
        }
        else {
          $loggerMessage .= ' ' . $arg;
          $arguments[$arg] = serialize($value);
        }
      }
    }

    if (!isset($this->io)) {
      $message = $message .
        ($entity_available ? '<br>@entity_name<br>' : '') .
        $loggerMessage;
      \Drupal::logger('drd')->{$severity}($message, $arguments);
    }
    else {
      $this->io->{$this->ioCallback($severity)}(t($message, $arguments)->render());
    }
  }

  public function debug($message, $args = []) {
    if (!$this->debug) {
      return;
    }
    $this->log('debug', $message, $args);
  }

  private function ioCallback($severity) {
    switch ($severity) {
      case 'emergency':
      case 'alert':
      case 'critical':
      case 'error':
        return 'error';

      case 'warning':
        return 'warning';

      case 'notice':
      case 'info':
      case 'debug':
      default:
        return 'note';

    }
  }

}
