<?php

namespace Drupal\drd\Plugin\Action;

use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a 'MaintenanceMode' action.
 *
 * @Action(
 *  id = "drd_action_maintenance_mode",
 *  label = @Translation("Maintenance Mode"),
 *  type = "drd_domain",
 * )
 */
class MaintenanceMode extends BaseEntityRemote implements BaseConfigurableInterface {

  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->arguments['mode'] = 'getStatus';
  }

  /**
   * @inheritdoc
   */
  public function executeAction($domain) {
    $response = parent::executeAction($domain);
    if ($response) {
      if ($this->arguments['mode'] == 'getStatus') {
        $domain->cacheMaintenanceMode($response['data']);
      }
      return TRUE;
    }
    return FALSE;
  }

  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['drd_action_maintenance_mode'] = array(
      '#type' => 'select',
      '#title' => t('Mode'),
      '#default_value' => 'getStatus',
      '#options' => [
        'getStatus' => t('get maintenance mode'),
        'on' => t('turn on maintenance mode'),
        'off' => t('turn off maintenance mode'),
      ],
    );
    return $form;
  }

  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {}

  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->arguments['mode'] = $form_state->getValue('drd_action_maintenance_mode');
  }

}
