<?php

namespace Drupal\drd\Plugin\Block;

use Drupal\Core\Url;

abstract class WidgetEntities extends WidgetBase {

  abstract protected function type();

  protected $accessView = FALSE;
  protected $accessCreate = FALSE;

  /**
   * @inheritDoc
   */
  public function __construct(array $configuration, $plugin_id, array $plugin_definition, $context = NULL) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $context);
    $this->accessView = \Drupal::currentUser()->hasPermission('drd.view published ' . $this->type() . ' entities');
    $this->accessCreate = \Drupal::currentUser()->hasPermission('drd.add ' . $this->type() . ' entities');
  }

  protected function message() {
    $active = \Drupal::database()->select('drd_' . $this->type(), 'h')
      ->condition('h.status', 1)
      ->countQuery()
      ->execute()
      ->fetchField();
    $inactive = \Drupal::database()->select('drd_' . $this->type(), 'h')
      ->condition('h.status', 0)
      ->countQuery()
      ->execute()
      ->fetchField();

    $name = $this->t('Create another ' . $this->type());
    if ($active == 0) {
      $message = $this->t('You currently have no active ' . $this->type() . ' in DRD!');
      $name = $this->t('Create your first ' . $this->type());
    }
    else {
      $message = $this->t('<p class="message">You have %count active ' . $this->type() . 's.</p>', ['%count' => $active]);
      if ($this->accessView) {
        $message .= $this->t('<p>Get all the details in your <a href="@list">' . $this->type() . ' list</a>.',
          ['@list' => (new Url('entity.drd_' . $this->type() . '.collection'))->toString()]);
      }
    }
    if ($inactive > 0) {
      $message = $this->t('<p>In addition, there are %count inactive ' . $this->type() . 's available as well.</p>', ['%count' => $inactive]);
    }

    if ($this->accessCreate && $this->type() != 'domain') {
      $message .= '<p class="action"><span class="button">' . \Drupal::linkGenerator()
          ->generate($name, new Url('entity.drd_' . $this->type() . '.add_form')) . '</span></p>';
    }

    return $message;
  }

}
