<?php

namespace Drupal\drd\Plugin\Update\Build;

use Drupal\Core\Form\FormStateInterface;
use Psy\Util\Json;

/**
 * Provides a composer based update build plugin.
 *
 * @Update(
 *  id = "composer",
 *  admin_label = @Translation("Composer"),
 * )
 */
class Composer extends Base {

  protected function implicitPatching() {
    return TRUE;
  }

  /**
   * @inheritDoc
   */
  public function defaultConfiguration() {
    return [
        'extra command' => '',
      ] + parent::defaultConfiguration();
  }

  /**
   * @inheritDoc
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $element = parent::buildConfigurationForm($form, $form_state);

    $element['extra_command'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Extra composer command'),
      '#default_value' => $this->configuration['extra command'],
    ];

    return $element;
  }

  /**
   * @inheritDoc
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['extra command'] = $this->getFormValue($form_state, 'extra_command');
  }

  /**
   * @inheritDoc
   */
  public function build($storage, $releases) {
    $json_filename = $storage->getWorkingDirectory() . DIRECTORY_SEPARATOR . 'composer.json';
    $lock_filename = $storage->getWorkingDirectory() . DIRECTORY_SEPARATOR . 'composer.lock';
    $hashes = [
      'json' => @hash_file('md5', $json_filename),
      'lock' => @hash_file('md5', $lock_filename),
    ];
    $composer = \Drupal\Component\Serialization\Json::decode(file_get_contents($json_filename));

    foreach ($releases as $release) {
      $name = ($release->getProjectType() == 'core') ?
        'core' :
        $release->getMajor()->getProject()->getName();
      $composer['require']['drupal/' . $name] = $release->getReleaseVersion();
    }

    file_put_contents($json_filename, Json::encode($composer, JSON_PRETTY_PRINT));

    if ($this->shell($storage, 'composer update --no-dev --no-progress --no-interaction')) {
      throw new \Exception('Composer failed.');
    }
    if (!empty($this->configuration['extra command'])) {
      if ($this->shell($storage, 'composer ' . $this->configuration['extra command'])) {
        throw new \Exception('Composer extra command failed.');
      }
    }
    $this->changed = (
      $hashes['json'] != @hash_file('md5', $json_filename) ||
      $hashes['lock'] != @hash_file('md5', $lock_filename)
    );

    return $this;
  }

}
