<?php

namespace Drupal\drd\Plugin\views\field;

use Drupal\Core\Render\Markup;
use Drupal\drd\Entity\ReleaseInterface;
use Drupal\drd\UpdateProcessor;
use Drupal\views\Plugin\views\field\Standard;
use Drupal\views\ResultRow;

/**
 * A handler to display the project update status.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("drd_update_status")
 */
class UpdateStatus extends Standard {
  /**
   * @inheritDoc
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    $this->realField = 'updatestatus';
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  private function statusToHtml($release, $status) {
    $statuses = UpdateProcessor::getStatuses();
    $type = $statuses['status'][$status]['type'];
    $title = $statuses['type'][$type]['title'];

    if ($release instanceof ReleaseInterface) {
      $class = 'drd-update-info';
      $major = $release->getMajor();
      $recommended = $major->getRecommendedRelease();
      $output = '<span class="drd-icon">&nbsp;</span><div class="label">' . $title . '</div>';
      if (!empty($recommended) && $recommended->id() != $release->id()) {
        $link_recommended = \Drupal::linkGenerator()
          ->generate($recommended->getVersion(), $recommended->getReleaseLink());
        $link_download = \Drupal::linkGenerator()
          ->generate($this->t('Download'), $recommended->getDownloadLink());
        $output .=
          '<div class="release">' . $link_recommended . '</div>' .
          '<div class="download">' . $link_download . '</div>';
      }
    }
    else {
      $class = 'drd-update-status';
      $output = substr($title, 0, 3);
    }

    return '<div class="' . $class . ' ' . $type . '">' . $output . '</div>';
  }

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
    $status = $this->getValue($values);
    if (empty($status)) {
      return '';
    }
    $release = $values->_entity;

    $output = '';
    foreach (explode(',', $status) as $value) {
      $output .= $this->statusToHtml($release, $value);
    }

    return Markup::create($output);
  }

}
