<?php

namespace Drupal\drd;

use Drupal\Component\Serialization\Json;

/**
 * Class SshCommand.
 *
 * @package Drupal\drd
 */
class SshCommand {

  /**
   * @var \Drupal\drd\Entity\DomainInterface
   */
  protected $domain;

  /**
   * @var SshInterface
   */
  protected $connection;

  /**
   * @var string
   */
  protected $command = '';

  /**
   * @param \Drupal\drd\Entity\DomainInterface $domain
   * @return $this
   */
  public function setDomain($domain) {
    $this->domain = $domain;
    $this->initConnection();
    return $this;
  }

  /**
   * @param string $command
   * @return $this
   */
  public function setCommand($command) {
    $this->command = $command;
    return $this;
  }

  /**
   * @return string
   */
  public function getOutput() {
    return $this->connection->getOutput();
  }

  /**
   * @return string
   */
  public function getJsonOutput() {
    try {
      return Json::decode($this->connection->getOutput());
    }
    catch (\Exception $ex) {
      return FALSE;
    }
  }

  /**
   * @return bool
   */
  public function execute() {
    return $this->connection->exec($this->command);
  }

  private function initConnection() {
    if (empty($this->domain->getCore()->getHost()->get('ssh2')->value)) {
      throw new \Exception('SSH for this host is disabled.');
    }

    $settings = $this->domain->getCore()->getHost()->getSshSettings();

    if (!empty($settings['host'])) {
      $host = $settings['host'];
    }
    else {
      $host = $this->domain->get('domain');
    }

    if (!function_exists('ssh2_connect')) {
      $this->connection = new SshPhp(
        $host,
        $settings['port'],
        $settings['auth']['mode'],
        $settings['auth']['username'],
        $settings['auth']['password'],
        $settings['auth']['file_public_key'],
        $settings['auth']['file_private_key'],
        $settings['auth']['key_secret']
      );
    }
    else {
      $this->connection = new SshLibSec(
        $host,
        $settings['port'],
        $settings['auth']['mode'],
        $settings['auth']['username'],
        $settings['auth']['password'],
        $settings['auth']['file_public_key'],
        $settings['auth']['file_private_key'],
        $settings['auth']['key_secret']
      );
    }

    if (!$this->connection->login()) {
      throw new \Exception('SSH authentication failed.');
    }
  }

}
