<?php

namespace Drupal\drd\Update;

/**
 * Defines the required interface for all DRD Update Storage plugins.
 */
interface PluginStorageInterface extends PluginInterface {

  /**
   * @param PluginBuildInterface $build
   * @param PluginProcessInterface $process
   * @param PluginTestInterface $test
   * @param PluginDeployInterface $deploy
   * @param PluginFinishInterface $finish
   *
   * @return $this
   */
  public function stepPlugins($build, $process, $test, $deploy, $finish);

  /**
   * @param \Drupal\drd\Entity\CoreInterface $core
   * @param \Drupal\drd\Entity\ReleaseInterface[] $releases
   * @param bool $dry
   * @param bool $showlog
   *
   * @return bool|string
   */
  public function execute($core, $releases, $dry, $showlog);

  /**
   * @param string|string[] $log
   *
   * @return $this
   */
  public function log($log);

  /**
   * @return \Drupal\drd\Entity\CoreInterface
   */
  public function getCore();

  /**
   * @return string
   */
  public function getDrupalDirectory();

  /**
   * @return string
   */
  public function getWorkingDirectory();

  /**
   * @return PluginBuildInterface
   */
  public function getBuildPlugin();

  /**
   * @return PluginProcessInterface
   */
  public function getProcessPlugin();

  /**
   * @return PluginTestInterface
   */
  public function getTestPlugin();

  /**
   * @return PluginDeployInterface
   */
  public function getDeployPlugin();

  /**
   * @return PluginFinishInterface
   */
  public function getFinishPlugin();

  /**
   * Determine and prepare a temporary working directory which will be used
   * during the update process and cleaned up at the end of it. Such a temp
   * directory is useful for workflows where you checkout a working copy from
   * a repository. However, if you already have a local working copy in the
   * file system accessible to the DRD web server user, then overwrite this
   * method in your plugin and return that directory yourself.
   *
   * @return $this
   */
  public function setWorkingDirectory();

  /**
   * @return $this
   */
  public function prepareWorkingDirectory();

  /**
   * @return $this
   */
  public function saveWorkingDirectory();

}
